@@ -13,8 +13,9 @@ use strict;
 use IO::File qw(!/^SEEK/);
 use POSIX;
 use Fcntl;
+use File::Basename;
 use HTML::Mason::Config;
-use Date::Manip;
+use HTML::Mason::Tools qw(date_delta_to_secs);
 
 sub access_data_cache
 {
@@ -25,6 +26,7 @@ sub access_data_cache
     #
     my $cacheFile = $options{cache_file} || die "cache: must specify cache file";
     my $physFile = $cacheFile . $HTML::Mason::DBM_FILE_EXT;
+    my $tieClass = ($options{tie_class} || $HTML::Mason::DEFAULT_CACHE_TIE_CLASS || die "no tie class defined!");
     my $action = $options{action} || 'retrieve';
     my $key = $options{key} || 'main';
     my $memCache = $options{memory_cache};
@@ -41,7 +43,7 @@ sub access_data_cache
 	#
 	# Validate parameters
 	#
-	if (my @invalids = grep(!/^(expire_(at|next|in)|action|key|value|memory_cache|cache_file)$/,keys(%options))) {
+	if (my @invalids = grep(!/^(expire_(at|next|in)|action|key|value|memory_cache|cache_file|tie_class)$/,keys(%options))) {
 	    die "cache: invalid parameter '$invalids[0]' for action '$action'\n";
 	}
 	
@@ -49,57 +51,41 @@ sub access_data_cache
 	# Determine expiration time if expire flag given
 	#
 	if (exists($options{expire_at})) {
-	    $expireTime = ParseDate($options{expire_at});
-	    die "cache: invalid expire_at value '$options{expire_at}' - must be a valid date\n" if !$expireTime;
+	    $expireTime = $options{expire_at};
+	    die "cache: invalid expire_at value '$options{expire_at}' - must be a numeric time value\n" if $expireTime !~ /^[0-9]+$/;
 	} elsif (exists($options{expire_next})) {
 	    my $term = $options{expire_next};
+	    my ($sec,$min,$hour) = localtime(time);
+	    my $deltaTime;
 	    if ($term eq 'hour') {
-		my $hour = UnixDate('now','%H');
-		my $nextHour = ($hour+1)%24;
-		$expireTime = Date_GetNext('now',undef,1,"$nextHour:00:01");
+		$deltaTime = 60*(60-$min)+(60-$sec);
 	    } elsif ($term eq 'day') {
-		$expireTime = Date_GetNext('now',undef,1,'00:00:01');
+		$deltaTime = 3600*(24-$hour)+60*(60-$min)+(60-$sec);
 	    } else {
 		die "cache: invalid expire_next value '$term' - must be 'hour' or 'day'\n";
 	    }
+	    $expireTime = time() + $deltaTime;
 	} elsif (exists($options{expire_in})) {
 	    my $delta = $options{expire_in};
-	    $delta = "+".$delta if substr($delta,0,1) ne '+';
-	    $delta = ParseDateDelta($delta);
-	    die "cache: invalid expire_in value '$options{expire_in}' - must be a valid date delta\n" if !$delta;
-	    $expireTime = ParseDate(scalar(DateCalc('now',$delta)));
+	    my $deltaTime = eval(date_delta_to_secs($delta));
+	    die "cache: invalid expire_in value '$options{expire_in}' - must be of the form <num><unit>, where <unit> is one of second, minute, hour, date, week or an abbreviation thereof\n" if !$deltaTime;
+	    $expireTime = time() + $deltaTime;
 	}
-	$expireTime = UnixDate($expireTime,'%s');
 
 	#
-	# Create dump version of store value
-	# MS 7/16/98: No longer needed under MLDBM
-	#	my $d = new Data::Dumper ([$options{value}]);
-	#	$d->Indent(0);
-	#	$d->Purity(1);
-	#	my $dump = $d->Dumpxs();
-	#	$dump =~ s/\$VAR1\s*=\s*/return /;
+	# Try to get lock on cache lockfile. If not possible, return.
 	#
-	# Create DB file if necessary
-	# MS 7/16/98: Shouldn't need the following if we trap tie later
-	#if (!-e $physFile) {
-	#    my %new;
-	#    tie (%new, 'MLDBM', $cacheFile, O_RDWR|O_CREAT, 0664);
-	#    untie(%new);
-	#}
-	#die "cache: cannot create cache file '$cacheFile'\n"
-	#	if (!-f $physFile);
-
-	# Create DB file if necessary
-	tie (%out, 'MLDBM', $cacheFile, O_RDWR|O_CREAT, 0664)
-		or die "cache: cannot create/open cache file '$cacheFile'\n";
-
-	#
-	# Try to get lock on DB file. If not possible, return.
-	#
-	my $lockfh = new IO::File ">>$physFile"
-	   or die "cache: cannot open cache file '$physFile' for locking\n";
+	my ($base,$lockdir) = fileparse($physFile);
+	$lockdir .= "locks";
+	mkpath($lockdir,0,0755) if (!-d $lockdir);
+	my $lockfile = "$lockdir/$base.lock";
+	my $lockfh = new IO::File ">>$lockfile"
+	   or die "cache: cannot open lockfile '$lockfile' for locking\n";
 	return unless &get_lock($lockfh);
+    
+	# Tie to DB file
+	tie (%out, $tieClass, $cacheFile, O_RDWR|O_CREAT, 0664)
+		or die "cache: cannot create/open cache file '$cacheFile'\n";
 
 	if ($action eq 'store') {
 	    $out{"$key.contents"} = $options{value};
@@ -116,7 +102,6 @@ sub access_data_cache
 		delete($memCache->{$path}->{$key});
 	    }
 	}
-	# this line causes a bizarre untie/eval bug
 	untie(%out);
 	$lockfh->close();
 
@@ -131,7 +116,7 @@ sub access_data_cache
 	#
 	# Validate parameters
 	#
-	if (my @invalids = grep(!/^(expire_(if|if_older_than)|action|key|memory_cache|cache_file)$/,keys(%options))) {
+	if (my @invalids = grep(!/^(expire_if|action|key|memory_cache|cache_file|tie_class)$/,keys(%options))) {
 	    die "cache: invalid parameter '$invalids[0]' for action '$action'\n";
 	}
 	
@@ -150,7 +135,7 @@ sub access_data_cache
 	#
 	if ($fileLastModified > $mem->{lastUpdated}) {
 	    my %in;
-	    tie (%in, 'MLDBM', $cacheFile, O_RDONLY, 0);
+	    tie (%in, $tieClass, $cacheFile, O_RDONLY, 0);
 
 	    #
 	    # If entry has been modified since we last updated, read
@@ -175,10 +160,6 @@ sub access_data_cache
 	    my $sub = $options{expire_if};
 	    return undef if (&$sub($mem->{lastModified}));
 	}
-	if (exists($options{expire_if_older_than})) {
-	    my $compareTime = $options{expire_if_older_than};
-	    return undef if ParseDate(scalar(localtime($mem->{lastModified}))) le $compareTime;
-	}
 	return $mem->{contents};
     } else {
 	die "cache: bad action '$options{action}': must be one of 'store', 'retrieve', or 'expire'\n";
